<?php

namespace Drupal\Tests\widget_instance\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\widget_instance\Entity\WidgetInstance;
use Drupal\widget_type\Entity\WidgetType;

/**
 * FunctionalJavascript tests for WidgetInstanceViewBuilder.
 *
 * @group widget_instance
 * @coversDefaultClass \Drupal\widget_instance\WidgetInstanceViewBuilder
 */
class WidgetInstanceViewBuilderTest extends WebDriverTestBase {

  private const BOILERPLATE_WIDGET_DIRECTORY = 'https://static.drupal.org/widget-registry-sandbox/widget-boilerplate/v1';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['widget_instance', 'widget_type'];

  /**
   * @covers ::view
   *
   * @SuppressWarnings(PHPMD.StaticAccess)
   */
  public function testView() {
    $widget_type = WidgetType::create([
      'name' => 'Boilerplate',
      'remote_widget_id' => 'widget-boilerplate',
      'remote_widget_version' => 'v1.10.2',
      'remote_widget_directory' => static::BOILERPLATE_WIDGET_DIRECTORY,
      'remote_widget_settings' => ['properties' => ['prop' => NULL]],
    ]);
    $widget_type->save();
    $entity = WidgetInstance::create([
      'name' => 'foo',
      'type' => $widget_type,
      'display_options' => ['prop' => 'Lorem Ipsum'],
    ]);
    $entity->save();
    $account = $this->drupalCreateUser([
      'access content',
      'view widget instance',
    ]);
    $this->drupalLogin($account);

    $this->drupalGet($entity->toUrl());
    $web_assert = $this->assertSession();
    // Ensure that a widget is loaded.
    $page = $this->getSession()->getPage();
    $has_spinner = $page->waitFor(10, function () use ($page) {
      return $page->find('css', 'div.spinner > div.double-bounce1');
    });
    self::assertFalse($has_spinner);
    // The boilerplate widget adds the text Welcome! asynchronously to the page.
    $web_assert->pageTextContainsOnce('Welcome!');
    $web_assert->buttonExists('Lorem Ipsum');
  }

}
